"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

var NodeModel =
/*#__PURE__*/
function () {
  function NodeModel(props) {
    var nodes = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    (0, _classCallCheck2["default"])(this, NodeModel);
    this.props = props;
    this.flatNodes = nodes;
  }

  (0, _createClass2["default"])(NodeModel, [{
    key: "setProps",
    value: function setProps(props) {
      this.props = props;
    }
  }, {
    key: "clone",
    value: function clone() {
      var _this = this;

      var clonedNodes = {}; // Re-construct nodes one level deep to avoid shallow copy of mutable characteristics

      Object.keys(this.flatNodes).forEach(function (value) {
        var node = _this.flatNodes[value];
        clonedNodes[value] = _objectSpread({}, node);
      });
      return new NodeModel(this.props, clonedNodes);
    }
  }, {
    key: "getChecked",
    value: function getChecked() {
      var optimisticToggle = this.props.optimisticToggle;
      var nodes = Object.values(this.flatNodes).map(function (_ref) {
        var category = _ref.category,
            checked = _ref.checked,
            checkState = _ref.checkState,
            id = _ref.id,
            label = _ref.label,
            properties = _ref.properties;
        return {
          category: category,
          checked: checked,
          checkState: checkState,
          id: id,
          label: label,
          properties: properties
        };
      }).filter(function (node) {
        return node.checkState === 1 || node.checkState === 2 && optimisticToggle;
      });
      return nodes;
    }
  }, {
    key: "getNode",
    value: function getNode(value) {
      return this.flatNodes[value];
    }
  }, {
    key: "flattenNodes",
    value: function flattenNodes(nodes) {
      var _this2 = this;

      var parent = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var depth = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

      if (!Array.isArray(nodes) || nodes.length === 0) {
        return;
      }

      var _this$props = this.props,
          disabled = _this$props.disabled,
          noCascade = _this$props.noCascade; // Flatten the `node` property for internal lookups

      nodes.forEach(function (node, index) {
        var isParent = _this2.nodeHasChildren(node);

        _this2.flatNodes[node.id] = {
          id: node.id,
          label: node.label,
          category: node.category,
          children: node.children,
          parent: parent,
          isParent: isParent,
          isLeaf: !isParent,
          showCheckbox: node.showCheckbox !== undefined ? node.showCheckbox : true,
          disabled: _this2.getDisabledState(node, parent, disabled, noCascade),
          treeDepth: depth,
          index: index,
          properties: node.properties
        };

        _this2.flattenNodes(node.children, node, depth + 1);
      });
    }
  }, {
    key: "nodeHasChildren",
    value: function nodeHasChildren(node) {
      return Array.isArray(node.children) && node.children.length > 0;
    }
  }, {
    key: "getDisabledState",
    value: function getDisabledState(node, parent, disabledProp, noCascade) {
      if (disabledProp) {
        return true;
      }

      if (!noCascade && parent.disabled) {
        return true;
      }

      return Boolean(node.disabled);
    }
  }, {
    key: "deserializeLists",
    value: function deserializeLists(lists) {
      var _this3 = this;

      var listKeys = ['checked', 'expanded']; // Reset values to false

      Object.keys(this.flatNodes).forEach(function (value) {
        listKeys.forEach(function (listKey) {
          _this3.flatNodes[value][listKey] = false;
        });
      }); // Deserialize values and set their nodes to true

      listKeys.forEach(function (listKey) {
        lists[listKey].forEach(function (value) {
          if (_this3.flatNodes[value] !== undefined) {
            _this3.flatNodes[value][listKey] = true;
          }
        });
      });
    }
  }, {
    key: "serializeList",
    value: function serializeList(key) {
      var _this4 = this;

      var list = [];
      Object.keys(this.flatNodes).forEach(function (value) {
        if (_this4.flatNodes[value][key]) {
          list.push(value);
        }
      });
      return list;
    }
  }, {
    key: "expandAllNodes",
    value: function expandAllNodes(expand) {
      var _this5 = this;

      Object.keys(this.flatNodes).forEach(function (value) {
        if (_this5.flatNodes[value].isParent) {
          _this5.flatNodes[value].expanded = expand;
        }
      });
      return this;
    }
  }, {
    key: "toggleChecked",
    value: function toggleChecked(node, isChecked, noCascade) {
      var _this6 = this;

      var flatNode = this.flatNodes[node.id];

      if (!flatNode.showCheckbox || node.disabled) {
        return this;
      }

      this.toggleNode(node.id, 'checked', isChecked);

      if (!flatNode.isLeaf && !noCascade) {
        flatNode.children.forEach(function (child) {
          _this6.toggleChecked(child, isChecked, noCascade);
        });
      }

      return this;
    }
  }, {
    key: "toggleNode",
    value: function toggleNode(nodeValue, key, toggleValue) {
      this.flatNodes[nodeValue][key] = toggleValue;
      return this;
    }
  }]);
  return NodeModel;
}();

var _default = NodeModel;
exports["default"] = _default;